# $Id: spear_wrapper.rb,v 1.3 2007-06-17 16:51:46 babic Exp $
#
# ParamILS wrapper for Spear theorem prover.

# Deal with inputs.
if ARGV.length < 5
	puts "spear_wrapper.rb is a wrapper for the Spear theorem prover."
	puts "Usage: ruby spear_wrapper.rb <instance_relname> <run_objective> <cutoff_time> <cutoff_length> <seed> <params to be passed on>."
	exit -1
end

input_file = ARGV[0]
optimum = ARGV[1].to_f
timeout = ARGV[2].to_i
cutoff_length = ARGV[3].to_i
seed = ARGV[4].to_i

# By default, ParamILS builds parameters as -param, but Spear requires
# --param. The following line fixes that.
tmpparams = ARGV[5...ARGV.length].collect{|x| x.sub(/^-sp/, "--sp")}

# Concatenate all params.
paramstring = tmpparams.join(" ")

# Build algorithm command and execute it.
cmd = "Spear --nosplash --time #{paramstring} --dimacs #{input_file} --tmout #{timeout} --seed #{seed}"
# Use this line for modular arithmetic
#cmd = "Spear --nosplash --time #{paramstring} --sf #{input_file} --tmout #{timeout} --seed #{seed}"

tmp_file = "spear_output#{rand}.txt"
exec_cmd = "#{cmd} > #{tmp_file}"

#STDERR.puts "Calling: #{exec_cmd}"
system exec_cmd

#=== Parse algorithm output to extract relevant information for ParamILS.
solved = nil
runtime = nil

File.open(tmp_file){|file|
	while line = file.gets
        if line =~ /UNSATISFIABLE/
            solved = "UNSAT"
		elsif line =~ /SATISFIABLE/
            solved = "SAT"
        elsif line =~ /(TIMEOUT)|(UNKNOWN)/
            solved = "TIMEOUT"
		end
		if line =~ /runtime (\d+\.\d+)/
			runtime = $1.to_f
		end
	end
}
File.delete(tmp_file)
puts "Result for ParamILS: #{solved}, #{runtime}, 0, 0, #{seed}"
